""" Authentication Class """
import bcrypt
import uuid


class Auth:
    """
    This class will be used to authenticate users based on the config/auth.py file
    """

    def __init__(self, request, auth_model=None):
        self.request = request

        if auth_model:
            self.auth_model = auth_model
        else:
            from config import auth
            self.auth_model = auth.AUTH['model']

    def user(self):
        """
        Returns the model specified in the auth.py configuration
        """
        try:
            if self.request.get_cookie('token'):
                return self.auth_model.where(
                    'remember_token', self.request.get_cookie('token')
                ).first()

            return False
        except Exception as exception:
            raise exception

        return None

    def login(self, name, password):
        """
        Login the user based on the parameters provided
        """
        auth_column = self.auth_model.__auth__
        try:
            model = self.auth_model.where(auth_column, name).first()

            if model and bcrypt.checkpw(bytes(password, 'utf-8'), bytes(model.password, 'utf-8')):
                remember_token = str(uuid.uuid4())
                model.remember_token = remember_token
                model.save()
                self.request.cookie('token', remember_token)
                return model

        except Exception as exception:
            raise exception

        return False

    def logout(self):
        self.request.cookie('token', '; expires=Thu, 01 Jan 1970 00:00:00 GMT', False)
        return self



""" Core of the IOC Container """

import inspect
from masonite.exceptions import MissingContainerBindingNotFound


class App():
    """
    Core of the Service Container. Performs bindings and resolving
    of objects to and from the container.
    """

    def __init__(self):
        self.providers = {}

    def bind(self, name, class_obj):
        """ Bind classes into the container with a key value pair """
        self.providers.update({name: class_obj})
        return self

    def make(self, name):
        """ Retreives a class from the container by key """
        if self.has(name):
            return self.providers[name]
        
        raise MissingContainerBindingNotFound("{0} key was not found in the container".format(name))
    
    def has(self, name):
        if name in self.providers:
            return True
        
        return False

    def helper(self):
        """ Adds a helper to create builtin functions """
        return self

    def resolve(self, obj):
        """
        Takes an object such as a function or class method and resolves it's 
        parameters from objects in the container
        """

        provider_list = []

        """
        Inspect all non annotation parameters, find them in the container
        and add them to the provider_list
        """
        for provider in inspect.signature(obj).parameters:
            if provider is not 'self' and provider not in inspect.getfullargspec(obj)[6]:
                provider_list.append(self.providers[provider])

        """
        Inspect all annotation parameters, find the object it annotates from the container
        and add the object to the provider_list
        """
        if inspect.getfullargspec(obj)[6]:
            provider_list = self.resolve_annotations(obj, provider_list)

        """
        Pass the provider list which contains all the dependencies found from the container
        into the parameters.
        """
        try:
            return obj(*provider_list)
        except TypeError:
            raise TypeError('Could not resolve the incorrect amount of objects from the container')

    def resolve_annotations(self, obj, provider_list):
        """
        Resolves class annotations (type hinted) parameters.
        This will retrieve the object from the container, not the key.
        """
        for parameter in inspect.signature(obj).parameters.values():
            if parameter.annotation:
                for provider, provider_class in self.providers.items():
                    if parameter.annotation == provider_class.__class__ or parameter.annotation == provider_class:
                        provider_list.append(provider_class)

        return provider_list



from masonite.routes import Get, Post, Delete, Patch, Put


def get(url, controller):
    """
    Shortcut for GET http
    """
    return Get().route(url, controller)


def post(url, controller):
    """
    Shortcut for POST http
    """
    return Post().route(url, controller)


def delete(url, controller):
    """
    Shortcut for DELETE http
    """
    return Delete().route(url, controller)


def put(url, controller):
    """
    Shortcut for Put http
    """
    return Put().route(url, controller)


def patch(url, controller):
    """
    Shortcut for PATCH http
    """
    return Patch().route(url, controller)



from masonite.contracts.BroadcastContract import BroadcastContract
from masonite.exceptions import DriverLibraryNotFound

try:
    import pusher
except ImportError:
    raise DriverLibraryNotFound(
        'Could not find the "pusher" library. Please pip install this library running "pip install pusher"')

class BroadcastPusherDriver(BroadcastContract):

    def __init__(self, BroadcastConfig):
        self.config = BroadcastConfig
        self.ssl_message = True
    
    def ssl(self, boolean):
        self.ssl_message = boolean
        return self

    def channel(self, channels, message, event='base-event'):
        pusher_client = pusher.Pusher(
            app_id=self.config.DRIVERS['pusher']['app_id'],
            key=self.config.DRIVERS['pusher']['client'],
            secret=self.config.DRIVERS['pusher']['secret'],
            ssl=self.ssl_message
        )

        if isinstance(message, str):
            message = {'message': message}  

        if isinstance(channels, list):
            for channel in channels:
                pusher_client.trigger(channel, event, message)
        else:
            pusher_client.trigger(channels, event, message)

        return message



""" An AppProvider Service Provider """
from masonite.provider import ServiceProvider
from masonite.request import Request
from masonite.routes import Route
from config import storage
from routes import web, api


class AppProvider(ServiceProvider):

    def register(self):
        self.app.bind('WebRoutes', web.ROUTES)
        self.app.bind('Response', None)
        self.app.bind('Storage', storage)
        self.app.bind('Route', Route())
        self.app.bind('Request', Request())
        self.app.bind('Container', self.app)

    def boot(self, Environ, Request, Route):
        self.app.bind('Headers', [])
        Route.load_environ(Environ)
        Request.load_environ(Environ).load_app(self.app)
